@@ -3,12 +3,17 @@ package s3
 import (
 	"bytes"
 	"context"
+	"encoding/xml"
 	"io/ioutil"
 	"net/http"
 	"path/filepath"
 	"reflect"
+	"strings"
 	"testing"
 
+	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/private/protocol/xml/xmlutil"
+	"github.com/aws/aws-sdk-go/service/s3"
 	"github.com/google/go-cmp/cmp"
 
 	corev1 "k8s.io/api/core/v1"
@@ -260,6 +265,7 @@ func TestServiceEndpointCanBeOverwritten(t *testing.T) {
 
 type tripper struct {
 	req           int
+	reqBodies     [][]byte
 	responseCodes []int
 }
 
@@ -268,6 +274,14 @@ func (r *tripper) RoundTrip(req *http.Request) (*http.Response, error) {
 		r.req++
 	}()
 
+	if req.Body != nil {
+		dt, err := ioutil.ReadAll(req.Body)
+		if err != nil {
+			return nil, err
+		}
+		r.reqBodies = append(r.reqBodies, dt)
+	}
+
 	code := http.StatusOK
 	if r.req < len(r.responseCodes) {
 		code = r.responseCodes[r.req]
@@ -423,3 +437,221 @@ func TestStorageManagementState(t *testing.T) {
 		})
 	}
 }
+
+func TestUserProvidedTags(t *testing.T) {
+	for _, tt := range []struct {
+		name          string
+		config        *imageregistryv1.Config
+		userTags      []configv1.AWSResourceTag
+		expectedTags  []*s3.Tag
+		responseCodes []int
+		infraName     string
+	}{
+		{
+			name:      "no user tags",
+			infraName: "test-infra",
+			expectedTags: []*s3.Tag{
+				{
+					Key:   aws.String("kubernetes.io/cluster/test-infra"),
+					Value: aws.String("owned"),
+				},
+				{
+					Key:   aws.String("Name"),
+					Value: aws.String("test-infra-image-registry"),
+				},
+			},
+			config: &imageregistryv1.Config{
+				Spec: imageregistryv1.ImageRegistrySpec{
+					Storage: imageregistryv1.ImageRegistryConfigStorage{
+						S3: &imageregistryv1.ImageRegistryConfigStorageS3{},
+					},
+				},
+			},
+		},
+		{
+			name:      "with user tags but no bucket",
+			infraName: "another-test-infra",
+			userTags: []configv1.AWSResourceTag{
+				{
+					Key:   "tag0",
+					Value: "value0",
+				},
+				{
+					Key:   "tag1",
+					Value: "value1",
+				},
+			},
+			expectedTags: []*s3.Tag{
+				{
+					Key:   aws.String("kubernetes.io/cluster/another-test-infra"),
+					Value: aws.String("owned"),
+				},
+				{
+					Key:   aws.String("Name"),
+					Value: aws.String("another-test-infra-image-registry"),
+				},
+				{
+					Key:   aws.String("tag0"),
+					Value: aws.String("value0"),
+				},
+				{
+					Key:   aws.String("tag1"),
+					Value: aws.String("value1"),
+				},
+			},
+			config: &imageregistryv1.Config{
+				Spec: imageregistryv1.ImageRegistrySpec{
+					Storage: imageregistryv1.ImageRegistryConfigStorage{
+						S3: &imageregistryv1.ImageRegistryConfigStorageS3{},
+					},
+				},
+			},
+		},
+		{
+			name:      "with user tags and already existing bucket",
+			infraName: "tinfra",
+			userTags: []configv1.AWSResourceTag{
+				{
+					Key:   "tag0",
+					Value: "value0",
+				},
+				{
+					Key:   "tag1",
+					Value: "value1",
+				},
+			},
+			expectedTags: []*s3.Tag{
+				{
+					Key:   aws.String("kubernetes.io/cluster/tinfra"),
+					Value: aws.String("owned"),
+				},
+				{
+					Key:   aws.String("Name"),
+					Value: aws.String("tinfra-image-registry"),
+				},
+			},
+			config: &imageregistryv1.Config{
+				Spec: imageregistryv1.ImageRegistrySpec{
+					Storage: imageregistryv1.ImageRegistryConfigStorage{
+						ManagementState: "Managed",
+						S3: &imageregistryv1.ImageRegistryConfigStorageS3{
+							Bucket: "a-bucket",
+						},
+					},
+				},
+			},
+		},
+		{
+			name:      "with user tags and creating provided bucket",
+			infraName: "tinfra",
+			userTags: []configv1.AWSResourceTag{
+				{
+					Key:   "tag0",
+					Value: "value0",
+				},
+				{
+					Key:   "tag1",
+					Value: "value1",
+				},
+			},
+			responseCodes: []int{http.StatusNotFound},
+			expectedTags: []*s3.Tag{
+				{
+					Key:   aws.String("kubernetes.io/cluster/tinfra"),
+					Value: aws.String("owned"),
+				},
+				{
+					Key:   aws.String("Name"),
+					Value: aws.String("tinfra-image-registry"),
+				},
+				{
+					Key:   aws.String("tag0"),
+					Value: aws.String("value0"),
+				},
+				{
+					Key:   aws.String("tag1"),
+					Value: aws.String("value1"),
+				},
+			},
+			config: &imageregistryv1.Config{
+				Spec: imageregistryv1.ImageRegistrySpec{
+					Storage: imageregistryv1.ImageRegistryConfigStorage{
+						ManagementState: "Managed",
+						S3: &imageregistryv1.ImageRegistryConfigStorageS3{
+							Bucket: "a-bucket",
+						},
+					},
+				},
+			},
+		},
+	} {
+		t.Run(tt.name, func(t *testing.T) {
+			builder := cirofake.NewFixturesBuilder()
+			builder.AddInfraConfig(&configv1.Infrastructure{
+				ObjectMeta: metav1.ObjectMeta{
+					Name: "cluster",
+				},
+				Status: configv1.InfrastructureStatus{
+					InfrastructureName: tt.infraName,
+					PlatformStatus: &configv1.PlatformStatus{
+						Type: configv1.AWSPlatformType,
+						AWS: &configv1.AWSPlatformStatus{
+							ResourceTags: tt.userTags,
+							Region:       "us-west-1",
+						},
+					},
+				},
+			})
+			builder.AddSecrets(&corev1.Secret{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      defaults.CloudCredentialsName,
+					Namespace: defaults.ImageRegistryOperatorNamespace,
+				},
+				Data: map[string][]byte{
+					"aws_access_key_id":     []byte("access_key_id"),
+					"aws_secret_access_key": []byte("secret_access_key"),
+				},
+			})
+			listers := builder.BuildListers()
+
+			drv := NewDriver(context.Background(), tt.config.Spec.Storage.S3, listers)
+			rt := &tripper{}
+			if len(tt.responseCodes) > 0 {
+				for _, code := range tt.responseCodes {
+					rt.AddResponse(code)
+				}
+			}
+			drv.roundTripper = rt
+
+			if err := drv.CreateStorage(tt.config); err != nil {
+				t.Errorf("unexpected err %q", err)
+				return
+			}
+
+			for _, body := range rt.reqBodies {
+				// ignore any other types of request.
+				if !strings.Contains(string(body), "Tagging") {
+					continue
+				}
+
+				buf := bytes.NewBuffer(body)
+				tagging := s3.Tagging{}
+				xmldec := xml.NewDecoder(buf)
+
+				if err := xmlutil.UnmarshalXML(&tagging, xmldec, ""); err != nil {
+					t.Fatalf("error decoding tagging request: %s", err)
+				}
+
+				if !reflect.DeepEqual(tagging.TagSet, tt.expectedTags) {
+					t.Fatalf(
+						"expected tags %+v, received %+v",
+						tt.expectedTags, tagging.TagSet,
+					)
+				}
+				return
+			}
+
+			t.Fatal("no request for tagging bucket found")
+		})
+	}
+}
